class GroundColorsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show]
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy]
  end

  def index
    @page = GroundColor.page params[:page]
    @page_size = GroundColor.page_size params[:page_size]
    @ground_colors = GroundColor.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = GroundColor.list_paginate(@page, @page_size)
      }
      format.json { render :json => @ground_colors.to_json(GroundColor.list_json_opt) }
    end
  end
  
  def show
    @ground_color = GroundColor.show(params[:id], [@user, @admin])
    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @ground_color.to_json(GroundColor.show_json_opt) }
    end
  end
  
  def new
    raise Pettanr::NotWork unless @author.working_panel
    @panel = Panel.edit(@author.working_panel, @author)
    @ground_color = GroundColor.new :panel_id => @panel.id, :code => params[:code].to_i
    @ground_color.supply_default

    respond_to do |format|
      format.html
      format.json { render :json => @ground_color.to_json(GroundColor.show_json_opt) }
    end
  end

  def edit
    @ground_color = GroundColor.show(params[:id], @author)
    @panel = Panel.edit(@ground_color.panel.id, @author)
    
    respond_to do |format|
      format.html
    end
  end

  def create
    raise Pettanr::NotWork unless @author.working_panel
    @panel = Panel.edit(@author.working_panel, @author)
    
    @ground_color = GroundColor.new 
    @ground_color.attributes = params[:ground_color]
    @ground_color.overwrite @panel.id
    
    respond_to do |format|
      if @ground_color.valid?
        if @ground_color.store @author
          flash[:notice] = I18n.t('flash.notice.created', :model => Panel.model_name.human)
          format.html { redirect_to @panel }
          format.json { render json: @panel.panel_elements_as_json, status: :created, location: @panel }
        else
          flash[:notice] = I18n.t('flash.notice.not_created', :model => GroundColor.model_name.human)
          format.html { render action: "new" }
          format.json { render json: @ground_color.errors, status: :unprocessable_entity }
        end
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => GroundColor.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @ground_color.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @ground_color = GroundColor.show(params[:id], @author)
    @ground_color.attributes = params[:ground_color]
    @panel = Panel.edit(@ground_color.panel.id, @author)
    @ground_color.overwrite @panel.id
    
    respond_to do |format|
      if @ground_color.store @author
        flash[:notice] = I18n.t('flash.notice.updated', :model => GroundColor.model_name.human)
        format.html { redirect_to @ground_color }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => GroundColor.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @ground_color.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @ground_color = GroundColor.show(params[:id], @author)
    @panel = Panel.edit(@ground_color.panel.id, @author)
    
    respond_to do |format|
      if @ground_color.remove @author
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => GroundColor.model_name.human)
        format.html { redirect_to @panel }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => GroundColor.model_name.human)
        format.html { redirect_to @ground_color }
        format.json { render json: @ground_color.errors, status: :unprocessable_entity }
      end
    end
  end
  
end
