class StorySheetsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show]
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = StorySheet.page params[:page]
    @page_size = StorySheet.page_size params[:page_size]
    @story_sheets = StorySheet.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = StorySheet.list_paginate(@page, @page_size)
      }
      format.json { render :json => @story_sheets.to_json(StorySheet.list_json_opt) }
    end
  end

  def show
    @story_sheet = StorySheet.show(params[:id], [@user, @admin])

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @story_sheet.to_json(StorySheet.show_json_opt) }
    end
  end
  
  def list
    @story_sheets = StorySheet.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @story_sheets }
    end
  end

  def browse
    @story_sheet = StorySheet.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @story_sheet }
    end
  end
  
  def new
    @story_sheet = StorySheet.new 
    @story_sheet.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @story_sheet.to_json(StorySheet.show_json_opt) }
    end
  end

  def edit
    @story_sheet = StorySheet.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  def create
    @story_sheet = StorySheet.new 
    @story_sheet.supply_default
    @story_sheet.attributes = params[:story_sheet]
    @story_sheet.overwrite @author
    @story = Story.edit(@story_sheet.story_id, @author) if @story_sheet.story_id
    @sheet = Sheet.show(@story_sheet.sheet_id, @author) if @story_sheet.sheet_id
    
    respond_to do |format|
      if @story_sheet.store
        flash[:notice] = I18n.t('flash.notice.created', :model => StorySheet.model_name.human)
        format.html { redirect_to play_story_path(@story) }
        format.json { render json: @story_sheet.to_json(StorySheet.show_json_opt) }
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => StorySheet.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @story_sheet.errors, status: :unprocessable_entity }
      end
    end
  end
  
  def update
    @story_sheet = StorySheet.edit(params[:id], @author)
    ot = @story_sheet.t
    @story_sheet.attributes = params[:story_sheet]
    @story_sheet.overwrite @author
    @story = Story.edit(@story_sheet.story_id, @author) if @story_sheet.story_id
    respond_to do |format|
      if @story_sheet.store ot
        flash[:notice] = I18n.t('flash.notice.updated', :model => StorySheet.model_name.human)
        format.html { redirect_to play_story_path(@story) }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => StorySheet.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @story_sheet.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @story_sheet = StorySheet.edit(params[:id], @author)
    @story = Story.edit(@story_sheet.story_id, @author) if @story_sheet.story_id
    respond_to do |format|
      if @story_sheet.destroy_and_shorten
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => StorySheet.model_name.human)
        format.html { redirect_to play_story_path(@story) }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => StorySheet.model_name.human)
        format.html { redirect_to story_sheet_path(@story_sheet) }
        format.json { render json: @story_sheet.errors, status: :unprocessable_entity }
      end
    end
  end
end
