module ApplicationHelper
  def full_url filename
    request.protocol + request.host_with_port + filename
  end
  
  def t_m(label)
    l = label.split('.')
    if l.size > 2
      label
    else
      if l.size == 1
        label.constantize.model_name.human
      else
        l.first.constantize.human_attribute_name(l.last)
      end
    end
  end
  
  def t_select_items(items)
    items.map {|i| [t(i[0]), i[1]] }
  end
  
  def t_selected_item(name, index)
    t(MagicNumber[name][index][0])
  end
  
  def icon_tag name, opt = {}
    title = opt[:object] ? (name.to_s + ' ID:' + opt[:object].id.to_s) : name
    size = opt[:size] ? opt[:size].to_i : 35
    tag :img, :src => asset_path(name.to_s.underscore + '.gif'), :width => size, :height => size, :alt => name.to_s + ' Icon', :title => title
  end
  
  def author_icon opt = {}
    icon_tag :Author, opt
  end
  
  def artist_icon opt = {}
    icon_tag :Artist, opt
  end
  
  def scroll_icon opt = {}
    icon_tag :Scroll, opt
  end
  
  def scroll_panel_icon opt = {}
    icon_tag :ScrollPanel, opt
  end
  
  def comic_icon opt = {}
    icon_tag :Comic, opt
  end
  
  def story_icon opt = {}
    icon_tag :Story, opt
  end
  
  def story_sheet_icon opt = {}
    icon_tag :StorySheet, opt
  end
  
  def sheet_icon opt = {}
    icon_tag :Sheet, opt
  end
  
  def sheet_panel_icon opt = {}
    icon_tag :Sheet, opt
  end
  
  def panel_icon opt = {}
    icon_tag :Panel, opt
  end
  
  def panel_editor_javascript_include_tags
    WritingFormat.enable_list().map {|wf|
#      javascript_include_tag wf.engine_name
      javascript_include_tag wf.engine_name + "/application"
    }.join("\n") + 
    SpeechBalloonTemplate.enable_list().map {|sbt|
      stylesheet_link_tag sbt.engine_name + "/application"
    }.join("\n") + 
    SpeechBalloonTemplate.enable_list().map {|sbt|
      javascript_include_tag sbt.engine_name + "/application"
    }.join("\n")
  end
  
  def configurations
    JSON.parse(<<HD
{
    "models": {
        "Panel": "panel",
        "panel": {
            "class_name": "Panel",
            "table_name": "panels",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "width": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "height": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "border": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 0
                    }
                },
                "caption": {
                    "type": "text",
                    "rules": {
                    }
                },
                "publish": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "author_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "PanelPicture": "panel_picture",
        "panel_picture": {
            "class_name": "PanelPicture",
            "table_name": "panel_pictures",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "panel_id": {
                    "type": "number",
                    "rules": {
                        "number": true
                    }
                },
                "picture_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "caption": {
                    "type": "text",
                    "rules": {}
                },
                "x": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "y": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "width": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "height": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "link": {
                    "type": "text",
                    "rules": {
                        "url": true
                    }
                },
                "z": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "t": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 0
                    }
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "SpeechBalloon": "speech_balloon",
        "speech_balloon": {
            "class_name": "SpeechBalloon",
            "table_name": "speech_balloons",
            "extend": "classname",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "panel_id": {
                    "type": "number",
                    "rules": {
                        "number": true
                    }
                },
                "speech_balloon_template_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "classname": {
                    "type": "text",
                    "rules": {
                        "required": true
                    }
                },
                "z": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "t": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 0
                    }
                },
                "settings": {
                    "type": "text",
                    "rules": {}
                },
                "caption": {
                    "type": "text",
                    "rules": {}
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "Balloon": "balloon",
        "balloon": {
            "class_name": "Balloon",
            "table_name": "balloons",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "speech_balloon_id": {
                    "type": "number",
                    "rules": {
                        "number": true
                    }
                },
                "system_picture_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "x": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "y": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "width": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "height": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "r": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "settings": {
                    "type": "text",
                    "rules": {}
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "Speech": "speech",
        "speech": {
            "class_name": "Speech",
            "table_name": "speeches",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "speech_balloon_id": {
                    "type": "number",
                    "rules": {
                        "number": true
                    }
                },
                "writing_format_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "content": {
                    "type": "text",
                    "rules": {}
                },
                "font_size": {
                    "type": "float",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 0
                    }
                },
                "text_align": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "range": [
                            0,
                            3
                        ]
                    }
                },
                "fore_color": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "range": [
                            0,
                            16777215
                        ]
                    }
                },
                "x": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "y": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "width": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "height": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "quotes": {
                    "type": "text",
                    "rules": {}
                },
                "settings": {
                    "type": "text",
                    "rules": {}
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "GroundPicture": "ground_picture",
        "ground_picture": {
            "class_name": "GroundPicture",
            "table_name": "ground_pictures",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "panel_id": {
                    "type": "number",
                    "rules": {
                        "number": true
                    }
                },
                "picture_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "caption": {
                    "type": "text",
                    "rules": {}
                },
                "repeat": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "range": [
                            0,
                            3
                        ]
                    }
                },
                "x": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "y": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "z": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "t": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 0
                    }
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "GroundColor": "ground_color",
        "ground_color": {
            "class_name": "GroundColor",
            "table_name": "ground_colors",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "panel_id": {
                    "type": "number",
                    "rules": {
                        "number": true
                    }
                },
                "caption": {
                    "type": "text",
                    "rules": {}
                },
                "code": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "range": [
                            0,
                            16777215
                        ]
                    }
                },
                "z": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 1
                    }
                },
                "t": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true,
                        "min": 0
                    }
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        },
        "WritingFormat": "writing_format",
        "writing_format": {
            "class_name": "WritingFormat",
            "table_name": "writing_formats",
            "attributes": {
                "id": {
                    "type": "number",
                    "primary_key": 1,
                    "rules": {
                        "number": true
                    }
                },
                "name": {
                    "type": "text",
                    "rules": {
                        "required": true
                    }
                },
                "classname": {
                    "type": "text",
                    "rules": {
                        "required": true
                    }
                },
                "caption": {
                    "type": "text",
                    "rules": {
                        "required": true
                    }
                },
                "system_picture_id": {
                    "type": "number",
                    "rules": {
                        "required": true,
                        "number": true
                    }
                },
                "settings": {
                    "type": "text",
                    "rules": {
                        "required": true
                    }
                },
                "created_at": {
                    "type": "datetime"
                },
                "updated_at": {
                    "type": "datetime"
                }
            }
        }
    },
    "elements": [
        {
            "name": "panel_picture",
            "path_name": "panel_pictures",
            "parts": []
        },
        {
            "name": "speech_balloon",
            "path_name": "circle_speech_balloon/speech_balloons",
            "parts": [
                "balloon",
                "speech"
            ]
        },
        {
            "name": "ground_picture",
            "path_name": "ground_pictures",
            "parts": []
        },
        {
            "name": "ground_color",
            "path_name": "ground_colors",
            "parts": []
        }
    ],
    "forms": {
        "Panel": "panel",
        "panel": {
            "attributes": {
                "caption": {
                    "column": "caption",
                    "type": "text",
                    "label": {},
                    "options": {
                        "row_break": true
                    }
                },
                "width": {
                    "column": "width",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "height": {
                    "column": "height",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    }
                },
                "border": {
                    "column": "border",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "publish": {
                    "column": "publish",
                    "type": "select",
                    "label": {},
                    "options": {
                        "source": "magic_number",
                        "key": "comic_visible_items",
                        "size": 5,
                        "row_break": true
                    }
                },
                "id": {
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "author_id": {
                    "column": "author_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            },
            "fields": [
                "caption",
                "width",
                "height",
                "border",
                "publish",
                "id",
                "author_id"
            ]
        },
        "PanelPicture": "panel_picture",
        "panel_picture": {
            "attributes": {
                "caption": {
                    "column": "caption",
                    "type": "text",
                    "label": {},
                    "options": {
                        "row_break": true
                    }
                },
                "x": {
                    "column": "x",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "y": {
                    "column": "y",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    }
                },
                "width": {
                    "column": "width",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    },
                    "helpers": {
                        "size": {
                            "path": "panels/size_helper",
                            "options": {
                                "class": "panel_picture_width_tool"
                            }
                        }
                    }
                },
                "height": {
                    "column": "height",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    },
                    "helpers": {
                        "size": {
                            "path": "panels/size_helper",
                            "options": {
                                "class": "panel_picture_height_tool"
                            }
                        }
                    }
                },
                "link": {
                    "column": "link",
                    "type": "text",
                    "label": {},
                    "options": {
                        "row_break": true
                    }
                },
                "id": {
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "panel_id": {
                    "column": "panel_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "picture_id": {
                    "column": "picture_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "z": {
                    "column": "z",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "t": {
                    "column": "t",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            },
            "fields": [
                "caption",
                "x",
                "y",
                "width",
                "height",
                "link",
                "id",
                "panel_id",
                "picture_id",
                "z",
                "t"
            ]
        },
        "speech_balloon": {
            "attributes": {
                "speech_balloon.id": {
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech_balloon.panel_id": {
                    "column": "panel_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech_balloon.speech_balloon_template_id": {
                    "column": "speech_balloon_template_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech_balloon.caption": {
                    "column": "caption",
                    "type": "text",
                    "label": {},
                    "options": {
                        "row_break": true
                    }
                },
                "speech_balloon.z": {
                    "column": "z",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech_balloon.t": {
                    "column": "t",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech_balloon.classname": {
                    "column": "classname",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech_balloon.settings": {
                    "column": "settings",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "balloon.id": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "balloon.speech_balloon_id": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "speech_balloon_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "balloon.system_picture_id": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "system_picture_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "balloon.r": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "r",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    },
                    "helpers": {
                        "tail_angle": {
                            "path": "panels/tail_angle_helper",
                            "options": {}
                        }
                    }
                },
                "balloon.x": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "x",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "balloon.y": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "y",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    }
                },
                "balloon.width": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "width",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "balloon.height": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "height",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    }
                },
                "balloon.settings": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "settings",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.id": {
                    "model": "speech",
                    "part": "speech",
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.speech_balloon_id": {
                    "model": "speech",
                    "part": "speech",
                    "column": "speech_balloon_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.x": {
                    "model": "speech",
                    "part": "speech",
                    "column": "x",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.y": {
                    "model": "speech",
                    "part": "speech",
                    "column": "y",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.width": {
                    "model": "speech",
                    "part": "speech",
                    "column": "width",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.height": {
                    "model": "speech",
                    "part": "speech",
                    "column": "height",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "speech.writing_format_id": {
                    "model": "speech",
                    "part": "speech",
                    "column": "writing_format_id",
                    "type": "select",
                    "label": {},
                    "options": {
                        "source": "model",
                        "model": "writing_format",
                        "method": "enable_list",
                        "caption": "caption"
                    }
                },
                "speech.font_size": {
                    "model": "speech",
                    "part": "speech",
                    "column": "font_size",
                    "type": "select",
                    "label": {},
                    "options": {
                        "source": "magic_number",
                        "key": "speech_font_size_items",
                        "row_break": true
                    }
                },
                "speech.text_align": {
                    "model": "speech",
                    "part": "speech",
                    "column": "text_align",
                    "type": "select",
                    "label": {},
                    "options": {
                        "source": "magic_number",
                        "key": "speech_text_align_items"
                    }
                },
                "speech.quotes": {
                    "model": "speech",
                    "part": "speech",
                    "column": "quotes",
                    "type": "text",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "speech.content": {
                    "model": "speech",
                    "part": "speech",
                    "column": "content",
                    "type": "text_area",
                    "label": {
                        "row_break": true
                    },
                    "options": {
                        "size": "45x5"
                    }
                },
                "speech.fore_color": {
                    "model": "speech",
                    "part": "speech",
                    "column": "fore_color",
                    "type": "hidden",
                    "label": {
                        "row_break": true
                    },
                    "options": {},
                    "helpers": {
                        "color": {
                            "path": "panels/color_helper",
                            "options": {},
                            "wrapper": "speech-fore_color-wrap"
                        }
                    }
                },
                "speech.settings": {
                    "model": "speech",
                    "part": "speech",
                    "column": "settings",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            }
        },
        "CircleSpeechBalloon": "circle_speech_balloon",
        "circle_speech_balloon": {
            "base": "speech_balloon",
            "attributes": {},
            "fields": [
                "speech_balloon.caption",
                "balloon.r",
                "balloon.x",
                "balloon.y",
                "balloon.width",
                "balloon.height",
                "balloon.id",
                "balloon.speech_balloon_id",
                "balloon.system_picture_id",
                "balloon.settings",
                "speech.writing_format_id",
                "speech.font_size",
                "speech.text_align",
                "speech.quotes",
                "speech.content",
                "speech.fore_color",
                "speech.id",
                "speech.speech_balloon_id",
                "speech.x",
                "speech.y",
                "speech.width",
                "speech.height",
                "speech.settings",
                "speech_balloon.id",
                "speech_balloon.panel_id",
                "speech_balloon.speech_balloon_template_id",
                "speech_balloon.classname",
                "speech_balloon.z",
                "speech_balloon.t",
                "speech_balloon.settings"
            ]
        },
        "PlainSpeechBalloon": "plain_speech_balloon",
        "plain_speech_balloon": {
            "base": "speech_balloon",
            "attributes": {
                "balloon.r": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "r",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            },
            "fields": [
                "speech_balloon.caption",
                "balloon.r",
                "balloon.x",
                "balloon.y",
                "balloon.width",
                "balloon.height",
                "balloon.id",
                "balloon.speech_balloon_id",
                "balloon.system_picture_id",
                "balloon.settings",
                "speech.writing_format_id",
                "speech.font_size",
                "speech.text_align",
                "speech.quotes",
                "speech.content",
                "speech.fore_color",
                "speech.id",
                "speech.speech_balloon_id",
                "speech.x",
                "speech.y",
                "speech.width",
                "speech.height",
                "speech.settings",
                "speech_balloon.id",
                "speech_balloon.panel_id",
                "speech_balloon.speech_balloon_template_id",
                "speech_balloon.classname",
                "speech_balloon.z",
                "speech_balloon.t",
                "speech_balloon.settings"
            ]
        },
        "SquareSpeechBalloon": "square_speech_balloon",
        "square_speech_balloon": {
            "base": "speech_balloon",
            "attributes": {
                "balloon.r": {
                    "model": "balloon",
                    "part": "balloon",
                    "column": "r",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            },
            "fields": [
                "speech_balloon.caption",
                "balloon.r",
                "balloon.x",
                "balloon.y",
                "balloon.width",
                "balloon.height",
                "balloon.id",
                "balloon.speech_balloon_id",
                "balloon.system_picture_id",
                "balloon.settings",
                "speech.writing_format_id",
                "speech.font_size",
                "speech.text_align",
                "speech.quotes",
                "speech.content",
                "speech.fore_color",
                "speech.id",
                "speech.speech_balloon_id",
                "speech.x",
                "speech.y",
                "speech.width",
                "speech.height",
                "speech.settings",
                "speech_balloon.id",
                "speech_balloon.panel_id",
                "speech_balloon.speech_balloon_template_id",
                "speech_balloon.classname",
                "speech_balloon.z",
                "speech_balloon.t",
                "speech_balloon.settings"
            ]
        },
        "GroundPicture": "ground_picture",
        "ground_picture": {
            "attributes": {
                "id": {
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "panel_id": {
                    "column": "panel_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "picture_id": {
                    "column": "picture_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "caption": {
                    "column": "caption",
                    "type": "text",
                    "label": {},
                    "options": {
                        "row_break": true
                    }
                },
                "repeat": {
                    "column": "repeat",
                    "type": "select",
                    "label": {},
                    "options": {
                        "source": "magic_number",
                        "key": "ground_picture_repeat_items",
                        "row_break": true
                    }
                },
                "x": {
                    "column": "x",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5
                    }
                },
                "y": {
                    "column": "y",
                    "type": "number",
                    "label": {},
                    "options": {
                        "size": 5,
                        "row_break": true
                    }
                },
                "z": {
                    "column": "z",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "t": {
                    "column": "t",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            },
            "fields": [
                "caption",
                "repeat",
                "x",
                "y",
                "z",
                "t",
                "id",
                "panel_id",
                "picture_id"
            ]
        },
        "GroundColor": "ground_color",
        "ground_color": {
            "attributes": {
                "id": {
                    "column": "id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "panel_id": {
                    "column": "panel_id",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "caption": {
                    "column": "caption",
                    "type": "text",
                    "label": {},
                    "options": {
                        "row_break": true
                    }
                },
                "code": {
                    "column": "code",
                    "type": "hidden",
                    "label": {
                        "row_break": true
                    },
                    "options": {
                        "row_break": true
                    },
                    "helpers": {
                        "color": {
                            "path": "panels/color_helper",
                            "options": {},
                            "wrapper": "ground_color-code-wrap"
                        }
                    }
                },
                "z": {
                    "column": "z",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                },
                "t": {
                    "column": "t",
                    "type": "hidden",
                    "label": {
                        "type": "hidden"
                    },
                    "options": {}
                }
            },
            "fields": [
                "id",
                "panel_id",
                "caption",
                "code",
                "z",
                "t"
            ]
        }
    },
    "locales": {
        "ja": {}
    }
}
HD
)
  end
end
